const BigInt & BigInt::operator +=(const BigInt & rhs)
// postcondition: returns value of bigint + rhs after addition
{

	 int sum;
	 int carry = 0;

	 int k;
	 int len = NumDigits();         // length of larger addend

	 if (IsPositive() != rhs.IsPositive())    // signs not the same, subtract
	 {
		  *this -= (-1 * rhs);
		  return *this;
	 }

	 // process both numbers until one is exhausted

	 if (len < rhs.NumDigits())
	 {
		  len = rhs.NumDigits();
	 }
	 for(k=0; k < len; k++)
	 {
		  sum = GetDigit(k) + rhs.GetDigit(k) + carry;
		  carry = sum / BASE;
		  sum = sum % BASE;

		  if (k < myNumDigits)
		  {
				ChangeDigit(k,sum);
		  }
		  else
		  {
				AddSigDigit(sum);
		  }
	 }
	 if (carry != 0)
	 {
		  AddSigDigit(carry);
	 }
	 return *this;
}


const BigInt & BigInt::operator -=(const BigInt & rhs)
// postcondition: returns value of bigint - rhs after subtraction
{
	 int diff;
	 int borrow = 0;

	 int k;
	 int len = NumDigits();

	 // signs opposite? then lhs - (-rhs) = lhs + rhs

	 if (IsNegative() != rhs.IsNegative())
	 {
		  *this +=(-1 * rhs);
		  return *this;
	 }
	 // signs are the same, check which number is larger
	 // and switch to get larger number "on top" if necessary
	 // since sign can change when subtracting
	 // examples: 7 - 3 no sign change,     3 - 7 sign changes
	 //          -7 - (-3) no sign change, -3 -(-7) sign changes
	 if (IsPositive() && (*this) < rhs ||
		  IsNegative() && (*this) > rhs)
	 {
		  *this = rhs - *this;
		if (IsPositive()) mySign = negative;
		else              mySign = positive;   // toggle sign
		  return *this;
	 }
	 // same sign and larger number on top

	 for(k=0; k < len; k++)
	 {
		  diff = GetDigit(k) - rhs.GetDigit(k) - borrow;
		  borrow = 0;
		  if (diff < 0)
		  {
				diff += 10;
				borrow = 1;
		  }
		  ChangeDigit(k,diff);
	 }
	 Normalize();
	 return *this;
}


const BigInt & BigInt::operator *=(const BigInt & rhs)
// postcondition: returns value of bigint * rhs after multiplication
{
	 // uses standard "grade school method" for multiplying

	if (IsNegative() != rhs.IsNegative())
	{
		mySign = negative;
	}
	else
	{
		mySign = positive;
	}

	 BigInt sum(0);                             // to accumulate sum
	 int k;
	 int len = rhs.NumDigits();                 // # digits in multiplier

	 for(k=0; k < len; k++)
	 {
		 sum += (*this) * rhs.GetDigit(k);          // k-th digit * self
		 (*this) *= 10;                             // add a zero
	 }
	 *this = sum;
	 return *this;
}


