#include <iostream.h>
#include <math.h>          // for sin, cos, sqrt
#include "rando.h"
#include "prompt.h"

class TwoDimWalk
{
  public:
	 TwoDimWalk(long int maxSteps,
               int size);      // # of steps, size of one step
    void First();              // take first step of walk
    bool IsDone();             // returns true if walk finished, else false
    void Next();               // take next step of random walk
    void Simulate();           // complete an entire random walk

    long int GetSteps();            // returns # of steps taken by molecule
    void GetPosition(double &x, // return x and y coordinates
                     double &y);

  private:
    void TakeStep();           // simulate one step of walk
    double myXcoord;           // current x coordinate
    double myYcoord;           // current y coordinate
	 long int mySteps;               // # of steps taken
    int myStepSize;            // size of step
    long int myMaxSteps;            // maximum # of steps allowed
};

TwoDimWalk::TwoDimWalk(long int maxSteps,int size)
// postcondition: all private data fields initialized     
{
    mySteps = 0;
    myMaxSteps = maxSteps;
    myStepSize = size;
}

void TwoDimWalk::TakeStep()
// postcondition: one step of random walk taken
{
    const double PI = 3.14159265;
    RandGen gen;                    // random number generator
    double randDirection = gen.RandReal() * 2 * PI;

    myXcoord += myStepSize * cos(randDirection);
    myYcoord += myStepSize * sin(randDirection);

    mySteps++;
}

void TwoDimWalk::First()
// postcondition: first step of random walk taken
{
    mySteps = 0;
    myXcoord = myYcoord = 0.0;
    TakeStep();
}

bool TwoDimWalk::IsDone()
// postcondition: returns true when random walk is finished
//                i.e., when # of steps taken == max. # of steps     
{
    return mySteps >= myMaxSteps;
}

void TwoDimWalk::Next()
// postcondition: next step in random walk simulated     
{
    TakeStep();
}

void TwoDimWalk::Simulate()
{
    for(First(); ! IsDone(); Next())
    {
        // simulation complete using iterator methods
    }
}

long int TwoDimWalk::GetSteps()
// postcondition: returns number of steps taken by molecule
{
	 return mySteps;
}

void TwoDimWalk::GetPosition(double & x, double & y)
// postcondition: return molecule's x coordinate
{
    x = myXcoord;
    y = myYcoord;
}

int main()
{
	 long int numSteps = PromptRange("enter # of random steps",1L,1000000L);
    int stepSize = PromptRange("size of one step",1,20);    
    int trials   = PromptRange("number of simulated walks",1,1000);
	 TwoDimWalk molecule(numSteps,stepSize);

    int k;
    double total = 0.0;
    double x,y;
    for(k=0; k < trials; k++)
    {
        molecule.Simulate();
        molecule.GetPosition(x,y);   // sum distance from origin of molecule
        total += sqrt(x * x + y * y);
    }
    cout << "average distance from origin = " << total/trials << endl;
    return 0;
}
