#include <iostream.h>
#include <fstream.h>              // for ifstream
#include <stdlib.h>               // for exit, atoi
#include "apstring.h"
#include "prompt.h"
#include "rando.h"
#include "apvector.h"

// reads file containing data for a cd in format below (one line/track)
//        track time (Min:Sec) Title of Track 
// example:
//         3:46    Bright Side Of The Road
//         2:36    Gloria
//         4:31    Moondance
//
// then randomly shuffle tracks and prints them
// treat each track as a line of data
//
// Owen Astrachan, August 4, 1994
// modified: September 27,1994, modified 7/13/96

struct TrackInfo
{
    apstring info;           // information about CD track
    int    trackNumber;    // the original track number
};
 
// function prototypes here

void ReadTracks(apvector<TrackInfo> & tracks, int & numTracks);
void PrintTracks(const apvector<TrackInfo> & tracks,int numTracks);
void ShuffleTracks(apvector<TrackInfo> & tracks,int numTracks);

int main()
{    
    const MAX_TRACKS = 20;
    int numTracks = 0;
    apvector<TrackInfo> allTracks(MAX_TRACKS);

    ReadTracks(allTracks,numTracks);
    ShuffleTracks(allTracks,numTracks);
    PrintTracks(allTracks,numTracks);
    
    return 0;
}

void ReadTracks(apvector<TrackInfo> & tracks, int & numTracks)
// postcondition: tracks initialized from user-specified file
//                numTracks = # entries in tracks     
{
    ifstream input;
    apstring filename = PromptString("enter name of data file: ");
    input.open(filename.c_str());
    
    if (input.fail())
    {
        cerr << "could not open file " << filename << endl;
        exit(0);
    }

    apstring trackinfo;                 // information about CD track
    numTracks = 0;                    // haven't read any tracks yet
    while (getline(input,trackinfo))  // read one line of file
    {
        tracks[numTracks].info = trackinfo;
        tracks[numTracks].trackNumber = numTracks;
        numTracks++;
    }
}

void PrintTracks(const apvector<TrackInfo> & tracks,int numTracks)
// precondition: numTracks = # valid entries in tracks
// postcondition: all track information printed     
{
    int k;
    for(k=0; k < numTracks; k++)
    {
         cout << "(" << tracks[k].trackNumber << ")\t"
              << tracks[k].info << endl;   
    }
}

void ShuffleTracks(apvector<TrackInfo> & tracks,int numTracks)
// precondition: numTracks = # of entries in tracks
// postcondition: entries in tracks have been randomly shuffled     
{
    RandGen gen;    // for random # generator
    int randTrack;
    TrackInfo temp;
    int k;    
    for(k=0; k < numTracks - 1; k++)
    {
        randTrack = gen.RandInt(k,numTracks-1);   // random track
        temp = tracks[randTrack];                 // swap entries
        tracks[randTrack] = tracks[k];
        tracks[k] = temp;
    }
}
