#include <iostream.h>
#include <fstream.h>
#include "apstring.h"
#include "worditer.h"
#include "prompt.h"

// compute authorship fingerprint using a class
// 6/28/96
// author: Owen Astrachan

class FingerPrint
{
  public:
    FingerPrint();                     // constructor
    void Calculate(apstring filename);   // calculate stats for text file
    void PrintStats();                 // print statistics
    
  private:
    int mySmall;                       // store counters
    int myMedium;                      // for different word lengths
    int myLarge;
};

FingerPrint::FingerPrint()
// postcondition: all fields initialized     
{
    mySmall = myMedium = myLarge = 0;
}

void FingerPrint::PrintStats()
// postcondition: statistics printed      
{
    int total = mySmall + myMedium + myLarge;
    
    cout << "\t # of small words = \t" << mySmall;
    cout << " =\t" << double(mySmall)*100/total << "%" << endl;

    cout << "\t # of medium words = \t" << myMedium;
    cout << " =\t" << double(myMedium)*100/total << "%" << endl;

    cout << "\t # of large words = \t" << myLarge;
    cout << " =\t" << double(myLarge)*100/total << "%" << endl;    
}


void FingerPrint::Calculate(apstring filename)
// precondition: filename = name of file that can be opened for reading
// postcondition: statistics have been calculated
{
    
    const int SMALL_SIZE = 4;      // mySmall words < this many letters
    const int LARGE_SIZE = 7;      // myLarge words > this many letters

    WordStreamIterator wstream;
    wstream.Open(filename);

    mySmall = myMedium = myLarge = 0;              // initialize counters
    for(wstream.First(); ! wstream.IsDone(); wstream.Next())
    {
        if (wstream.Current().length() < SMALL_SIZE)        mySmall++;
        else if (wstream.Current().length() > LARGE_SIZE)   myLarge++;
        else                                                myMedium++;
    }    
}

int main()
{
    apstring filename = PromptString("enter name of file: ");
    FingerPrint finger;
    
    finger.Calculate(filename);
    finger.PrintStats();
    
    return 0;
}
