#include <iostream.h>
#include <iomanip.h>
#include "clockt.h"

ClockTime::ClockTime(int secs, int mins, int hours)
  : mySeconds(secs), myMinutes(mins), myHours(hours)
// postcondition: all data fields initialized     
{
    // work done in initializer list
}

int ClockTime::Hours() const
// postcondition: return # of hours
{
    return myHours;
}

int ClockTime::Minutes() const
// postcondition: return # of minutes
{
    return myMinutes;
}

int ClockTime::Seconds() const
// postcondition: return # of seconds
{
    return mySeconds;
}

void ClockTime::Print(ostream & os) const
// postcondition: insert object in format h:m:s onto stream os
{
    char saveFill = os.fill();     // previous fill character    
    os.fill('0');                  // pad with zeros
    
    os << Hours() << ":"
       << setw(2) << Minutes() << ":"
       << setw(2) << Seconds();

    os.fill(saveFill);             // reset stream fill character
}

ostream & operator << (ostream & os, const ClockTime & ct)
// postcondition: inserts ct onto os, returns os
//                format is h:m:s     
{
    ct.Print(os);
    return os;
}

ClockTime & ClockTime::operator += (const ClockTime & ct)
// postcondition: add ct, return result (normalized for myMinutes, mySeconds)     
{
    mySeconds += ct.mySeconds;           // add mySeconds
    myMinutes += mySeconds / 60;         // overflow from secs to myMinutes
    mySeconds %= 60;                     // now between 0 and 59
    
    myMinutes += ct.myMinutes;           // add myMinutes
    myHours += myMinutes / 60;           // overflow from myMinutes to myHours
    myMinutes %=  60;                    // now between 0 and 59
    
    myHours += ct.myHours;
    
    return *this;
}

ClockTime operator + (const ClockTime & lhs, const ClockTime & rhs)
// postcondition: return lhs + rhs (normalized for myMinutes, mySeconds)
{
    ClockTime result(lhs);
    result += rhs;
    return result;
}


