#ifndef _CLOCKTIME_H
#define _CLOCKTIME_H

#include <iostream.h>

// class for manipulating "clock time", time given in hours, minutes, seconds
// class supports only construction, addition, Print() and output <<
//
// Owen Astrachan: written May 25, 1994
//                 modified Aug 4, 1994, July 5, 1996
//
// ClockTime(int secs = 0, int mins = 0, int hours = 0)
//                -- default values of 0 for all three
//
//      access functions
//
//      Hours()    -- returns # of hours in ClockTime object
//      Minutes()  -- returns # of minutes in ClockTime object
//      Seconds()  -- returns # of seconds in ClockTime object
//      Print()    -- print time in format h:m:s
//                    (with :, no space, zero padding)    
//
//      operators (for addition and output)
//
//      ClockTime & operator +=(const ClockTime & ct)
//      ClockTime operator +(const ClockTime & a, const ClockTime & b)
//
//      ostream & operator <<(ostream & os, const ClockTime & ct)
//           inserts ct into os, returns os, uses Print()

class ClockTime
{
  public:
    ClockTime(int secs = 0, int mins = 0, int hours = 0);
    
    int Hours()   const;        // returns # hours
    int Minutes() const;        // returns # minutes
    int Seconds() const;        // returns # seconds
    void Print(ostream & os)  const;   // prints ClockTime object
	
    ClockTime & operator +=(const ClockTime & ct);
    
  private:
    int mySeconds;           // constrained: 0-59    
    int myMinutes;           // constrained: 0-59
    int myHours;
};
// free functions, not member functions

ostream &  operator << (ostream & os, const ClockTime & ct);
ClockTime operator + (const ClockTime & lhs, const ClockTime & rhs);

#endif
