#include <iostream.h>
#include "dice.h"
#include "prompt.h"

// simulate multiple games of craps (rules from W. Weaver, Lady Luck, 1982
// Doubleday Press --- as given in Charles Whitney, Random Process in
// Physical Systems, 1990, Wiley Interscience)
//
// written 5/17/94, modified 6/21/95, modified 6/25/96

// prototypes

bool GetPoint(int point);
bool WinGame();
int RollTwo();

int main()
{
   long int k;
   int gamesWon = 0;
	long int simulations = PromptRange("enter # games to simulate",1L,1000000L);

   for(k=0; k < simulations; k++)
   {
       if (WinGame())
       {
           gamesWon++;
       }
   }
   cout << "# of games = " << simulations;
   cout << " # won = "     << gamesWon << " = ";
   cout << double(gamesWon)/simulations * 100 << "%" << endl;
   return 0;
}

bool WinGame()
// postcondition: return true if single game of craps won
//                return false if game lost
{
    int point = RollTwo();                     // initial point

    switch (point)
    {
      case 7:
      case 11:
        // cout << "natural winner!!" << endl;
        return true;
        break;
      case 2:
      case 3:
      case 12:
        // cout << "craps!! a loser" << endl;
        return false;
        break;
      default:
        return GetPoint(point);
    }    
}

bool GetPoint(int point)
// precondition: 2 <= point <= 12
// postcondition: returns true if point obtained (winner)
//                returns false if 'crapped out'
{
    int sum;
    do
    {
        sum = RollTwo();
    } while (sum != point && sum != 7);

    return (sum == point);
}

int RollTwo()
//postcondition: returns sum of rolling two dice
{
    Dice d(6);

    int first = d.Roll();
    int second = d.Roll();
//  cout << "rolling ... " << first << " " << second << endl;
    return first + second;
}
