#include "directry.h"
#include <string.h>
#include <dirent.h>
#include <errno.h>
#include <sys/stat.h>
#include <stdlib.h>             // fullpath in Turbo 4.5

const int PATH_SIZE = 500;
static char buf[PATH_SIZE];

//
// author: Owen Astrachan
// date: 10/15/93, ported to Dec stations on 11/8/93
// modified 1/18/96 to add Direntry constructor
//
// implementation of directory classes for reading info
// about directories.  See directory.h
//
//
//

DirEntry::DirEntry()
{
	//currently empty
}

DirEntry::DirEntry(const apstring & name)
   : myName(""),
     myPath(""),
     mySize(0),
     myIsDirectory(false)
{
    struct stat statEntry;
    char * bufPtr;
    if (0 == stat(name.c_str(),&statEntry))
    {
	myName = name;
	// in Unix use function realpath, in dos, use _fullpath (stdlib.h)
	bufPtr = realpath(const_cast<char *>(name.c_str()),buf);
	// bufPtr = _fullpath(buf,name.c_str(),PATH_SIZE);
	if (bufPtr != 0)
	{
	    myPath = buf;
	}
	mySize = statEntry.st_size;
	//myIsDirectory = S_ISDIR(statEntry.st_mode);
	 myIsDirectory = statEntry.st_mode & S_IFDIR;
	 }
}


DirEntry::~DirEntry()
{
	//currently empty
}

apstring DirEntry::Path() const
{
    return myPath;
}

apstring DirEntry::Name() const
{
    return myName; //dirEntry->d_name;
}

long int DirEntry::Size() const
{
    return mySize; //statEntry.st_size;
}

bool DirEntry::IsDir() const
{
    return myIsDirectory; //S_ISDIR(statEntry.st_mode);
}

void DirStream::Initialize(const apstring & s)
// postcondition: all private data initialized     
{
    myPath = s;                 // remember path to this directory

    if (myStream != 0 && !myIsClosed)
    {
	closedir(myStream);
    }
    myIsClosed = false;
    
    myStream = opendir(s.c_str());      // open the direcotry
#ifdef DEBUG
    if (myStream == 0)
    {
	if (errno == EMFILE) cout << "too many descriptors" << endl;
	else if (errno == ENFILE) cout << "table full" << endl;
	else cout << errno << endl;
    }
#endif
    myStatus = myStream != 0;   // check if open succeeded
}

DirStream::DirStream(const apstring & s)
   : myStream(0),
     myIsClosed(false)
// postcondition: initialized to directory whose path is s     
{
    Initialize(s);
}


DirStream::DirStream()
   : myStream(0),
     myIsClosed(false)
// postcondition: initialized to current directory     
{
    apstring self = getcwd(buf,PATH_SIZE);
    Initialize(self);
}

bool DirStream::fail()
{
    return ! myStatus || myStream == 0;
}


void DirStream::open(const apstring & s)
{
    Initialize(s);
}

DirStream::~DirStream()
{
    close();
}

void DirStream::close()
{
    if (myStream != NULL)
    {
	closedir(myStream);     // close up properly
        myIsClosed = true;
    }
}

int DirStream::Read(DirEntry & dir)
// postcondition: directory info stored in dir for 'next' file
//                returns 1 if read succeeds, else 0     
{
    char * myBuf = 0;
    struct stat statEntry;    
    struct dirent *entry = readdir(myStream);    // get dir entry

    if (entry != NULL)
    {
	dir.myName = entry->d_name;

	    // add / + name of dir to fullpath

	apstring path = myPath + DIR_SEPARATOR + dir.myName;
	myBuf = realpath(const_cast<char *>(path.c_str()),buf);
	// myBuf = _fullpath(buf,path.c_str(),PATH_SIZE);
	if (myBuf != 0)
	{
	    dir.myPath = buf;
	}
	dir.mySize = 0;
	dir.myIsDirectory = false;
	
	if (0 == stat(path.c_str(),&statEntry))
	{
	    dir.mySize = statEntry.st_size;
		 //dir.myIsDirectory = S_ISDIR(statEntry.st_mode);
		 dir.myIsDirectory = statEntry.st_mode & S_IFDIR;
		 myStatus = 1;
	    return myStatus;
	}
	// cerr << "stat failed " << path << endl;
	return myStatus = Read(dir);
    }
    else
    {
	return myStatus = 0;
    }
	// not reached

}

void DirStream::First()
{
    if (myStream != NULL)
    {
	rewinddir(myStream);
    }
    (void) GetEntry(myEntry);
}

void DirStream::Next()
{
    (void) GetEntry(myEntry);
}

DirEntry DirStream::Current()
{
    return myEntry;
}

bool DirStream::IsDone()
{
    return fail();
}


bool DirStream::GetEntry(DirEntry & dir)
// postcondition: directory info stored in dir for 'next' file
//                returns 1 if read succeeds, else 0     
{    

    if (myStream == 0)             // stream no good?
    {
	return false;
    }
    return Read(dir);
}





