#include <iostream.h>
#include <stdlib.h>               // for exit
#include <math.h>
#include "heart.h"
#include "dice.h"
#include "rando.h"

// implementation of heart monitor (cardioverter/defibrillator)
// Owen Astrachan
// written 9/8/93
// extensively modified 1/16/94
// now supports input from ifstream
// modified 6/95 to add IsBeating member function
// modified 6/96 to add external monitor

Heart::Heart(Monitor & m)
   : myFromFile(false),
     mySenseCount(0),
     myBeating(true),
     myMonitor(m)
// postcondition: heart is initialized to "sense itself"
//                all fields initialized     
{

}

void Heart::SetFile(const apstring & name)
// precondition: text file "name" can be open for reading
// postcondition: heart is set to read data from file "name"
{
    myFile.open(name.c_str());
    myBeating = false;
    myFromFile = true;
    
    if (myFile.fail())
    {
        cerr << "could not open file " << name << endl;
    }
    else
    {
        if (myFile >> mySense)
        {
            mySenseCount = 0;
            myBeating = true;
        }
        else
        {
            myBeating = false;
        }
    }
}

int Heart::Sense()
// precondition: if reading from file: file specified by myFile has
//               unread numerical data
// postcondition: next heart 'sense' is returned, also monitored
// exception: if int read fails, INT_MAX returned
// side-effect: mySenseCount updated
//              heart is monitored
{
    int value;

    if (myFromFile)             // reading sense from file?
    {
        if (IsBeating())
        {
            value = mySense;
            mySenseCount++;
            myFile >> mySense;
            if (myFile.fail())
            {
                myBeating = false;
            }
        }
        else
        {
            value = INT_MAX;
            myBeating = false;
        }
    }
    else                         // generating own sense
    {
        mySenseCount++; 
        value = SelfSense();
    }
    myMonitor.DisplaySense(itoa(value));
    return value;
}

int Heart::SelfSense()
// postcondition: returns a value representing electric signal
// side-effect:   heart is monitored     
{
    static RandGen gen;       // for random electrical fluctuations

    int value =  gen.RandInt(MIN_VOLT,MAX_VOLT);

    return value;
}
     

bool Heart::IsBeating()
// postcondition: returns true if heart is beating;
//                otherwise returns false
{
    return myBeating;
}

void Heart::Shock(ShockType s)
// postcondition: shock method printed     
{
    myMonitor.DisplayShock("Shocked after " + itoa(mySenseCount) + " senses");

    if (s == SpeedUp)
    {
        myMonitor.DisplayShock("SPEED UP");
    }
    else
    {
        myMonitor.DisplayShock("SLOW DOWN");
    }
}
