#include "list.h"

template <class Type>
List<Type>::List()
// postcondition: empty list constructed     
   : myCount(0),
     myBefore(0),
     myCurrent(0),
     myCurrentValid(false)
{
    myFirst = new TNode;                       // create header node
    myLast = myFirst;
}

template <class Type>
const List<Type> &
List<Type>::operator =(const List<Type> & list)
{

    if (this == &list) return *this;    // watch out for self assign
    
    Clear();
    
    myCurrentValid = false;   // don't copy iterator values
    myCurrent      = 0;
    myBefore       = 0;

    // append each value from list to myself, this makes me a copy of list
    
    for(list.First(); ! list.IsDone(); list.Next())
    {
        Append(list.Current());
    }
    return *this;
}


template <class Type>
List<Type>::List(const List<Type> & list)
// postcondition: *this is a copy of list
  : myCount(0),
    myBefore(0),
    myCurrent(0),
    myCurrentValid(false)    
{
    myFirst = new TNode;
    myLast = myFirst;
    
    for(list.First(); ! list.IsDone(); list.Next())
    {
        Append(list.Current());
    }
}

template <class Type>
List<Type>::~List()
{
    Clear();
    delete myFirst;   // delete header node
}

template <class Type>
void List<Type>::Clear()
// postcondition: all nodes returned to storage except header
//                list is empty     
{
    TNode * current = myFirst->next;   // after header node
    TNode * temp;
    while (current)
    {
        temp = current->next;
        delete current;
        current = temp;
    }
    myLast = myFirst;
    myFirst->next = 0;    // only header node
    myCount = 0;
}

template <class Type>
void List<Type>::Append(const Type & t)
// postcondition: new node containing t added to end of list     
{
    myLast->next = new TNode(t);
    myLast = myLast->next;
    myCount++;
}

template <class Type>
void List<Type>::Prepend(const Type & t)
// postcondition: new node containing t aded to front of list     
{
    // add new node after header node
    
    myFirst->next = new TNode(t,myFirst->next);  
    if (myLast == myFirst)        // if old list was empty
    {                             // the last shall be first
        myLast = myFirst->next;
    }
    myCount++;    
}

template <class Type>
Type List<Type>::Front() const
{
    if (myFirst->next)
    {
        return myFirst->next->info;
    }
    return myFirst->info;    // this is garbage
}

template <class Type>
Type List<Type>::Back() const
{
    return myLast->info;
}

template <class Type>
void List<Type>::ChopFront()
{
    TNode * temp = myFirst->next;
    if (temp != 0)
    {
        myFirst->next = temp->next;
        delete temp;
        myCount--;
    }
    // if list is empty,make myLast correct
    if (myFirst->next == 0)
    {
	myLast = myFirst;
    }    
}

template <class Type>
bool List<Type>::Contains(const Type & t) const
// postcondition: returns true if t contained in list, else false
{
    TNode * temp = myFirst->next;  // start at front, search
    while (temp)                   // linearly
    {
        if (temp->info == t) return true;
        temp = temp->next;
    }
    return false;
}

template <class Type>
void List<Type>::First() const
// postcondition: iterator set to first node of list     
{
    const_cast<List<Type> &>(*this).myBefore = myFirst;
    const_cast<List<Type> &>(*this).myCurrent = myBefore->next;
    const_cast<List<Type> &>(*this).myCurrentValid = true;


    /***** if your compiler doesn't support const_cast use lines below
	   
    ((List<Type> &)(*this)).myBefore = myFirst;
    ((List<Type> &)(*this)).myCurrent = myBefore->next;
    ((List<Type> &)(*this)).myCurrentValid = true;

    ************/
}

template <class Type>
bool List<Type>::IsDone() const
// postcondition: returns true if Current() access is ok
//                otherwise false
{
    if (!myCurrentValid)
    {
        const_cast<List<Type> &>(*this).myCurrentValid = true;
        const_cast<List<Type> &>(*this).myCurrent = myBefore->next;

	/***** if your compiler doesn't support const_cast use below
	       
        ((List<Type> &)(*this)).myCurrentValid = true;      
        ((List<Type> &)(*this)).myCurrent = myBefore->next;
	       
	************/
    }
    return myCurrent == 0;
}

template <class Type>
void List<Type>::Next() const
// precondition: ! IsDone()
// postcondition: iterator advanced to next item     
{
    if (!myCurrentValid)
    {
        const_cast<List<Type> &>(*this).myCurrentValid = true;
        const_cast<List<Type> &>(*this).myCurrent = myBefore->next;

	/***** if your compiler doesn't support const_cast use below
	       
        ((List<Type> &)(*this)).myCurrentValid = true;      
        ((List<Type> &)(*this)).myCurrent = myBefore->next;
	       
	************/	
    }
    else
    {
        const_cast<List<Type> &>(*this).myBefore = myCurrent;
        const_cast<List<Type> &>(*this).myCurrent = myCurrent->next;

	/***** if your compiler doesn't support const_cast use below
	       
        ((List<Type> &)(*this)).myBefore = myCurrent; 
        ((List<Type> &)(*this)).myCurrent = myCurrent->next;
	       
	************/		
    }
}

template <class Type>
Type& List<Type>::Current()
// precondition: ! IsDone()     
// postcondition: returns current element of list     
{
    if (myCurrent != 0 && myCurrentValid)       
    {
        return myCurrent->info; 
	 }
	 cerr << "illegal call of Current() in list class" << endl;
	 abort();
	 return myCurrent->info;   // not called, but avoid warnings
}

template <class Type>
const Type& List<Type>::Current() const
// precondition: ! IsDone()
// postcondition: returns current element of list     
{
    if (myCurrent != 0 && myCurrentValid)       
    {
        return myCurrent->info; 
	 }
    cerr << "illegal call of Current() in list class" << endl;
	 abort();
	 return myCurrent->info;   // not called, but avoid warnings
}

template <class Type>
void List<Type>::Delete()
// postcondition: delete current node, new current is after
//                old current     
{
    if (myCurrent != 0)
    {
        myBefore->next = myCurrent->next;   // link around current node
        if (myCurrent == myLast)            // update myLast if needed
        {
            myLast = myBefore;
        }
        delete myCurrent;                 
        myCurrent = 0;                      // invalidate current
        myCurrentValid = false;
        myCount--;
    }
}

template <class Type>
void List<Type>::InsertBefore(const Type & t)
// postcondition: t inserted before current item
{
    if (myCurrent != 0)
    {
        myBefore->next = new TNode(t,myCurrent);
        myCount++;      
    }
}


template <class Type>
int List<Type>::Size() const
// postcondition: returns # of elements in list
{
    return myCount;
}

template <class Type>
int List<Type>::Length() const
// postcondition: returns # of elements in list
{
    return myCount;
}

template <class Type>
void List<Type>::Print() const
{
    TNode * temp = myFirst->next;
    cout << "( ";
    while (temp)
    {
        cout << temp->info << " ";
        temp = temp->next;
    }
    cout << ")" << endl;
}
