#include <iostream.h>
#include "apstring.h"
#include "list.cpp"
#include "worditer.h"
#include "prompt.h"
#include "ctimer.h"
#include "strutils.h"

// Owen Astrachan, 7/1/96
// illustrates List class for tracking words and occurrences

class WordCounter
{
  public:
    WordCounter();                  // need default constructor for List
    WordCounter(apstring word);       // store word with count = 1
    void   Increment();             // bump the counter by 1
    int    GetCount() const;        // return word count
    apstring GetWord()  const;        // return word
    bool operator == (const WordCounter & rhs);
        
  private:
    
    apstring myWord;    // store word
    int myCount;      // # occurrences of word
};

// in Borland, << must be defined even if not called, so we'll
// define an operator << to satisfy the compiler
ostream& operator << (ostream & out, const WordCounter & wc)
{
	  out << wc.GetWord();
     return out;
}

void Update(List<WordCounter> & wordList, const WordCounter & wc);
void Print(const List<WordCounter> & wordList);
void MaxStats(const List<WordCounter> & wordList, apstring & maxWord,
              int & maxOccurrences);
int main()
{
    WordStreamIterator iter;
    apstring word;
    List<WordCounter> wordList;
    CTimer timer;    
    apstring filename = PromptString("enter file name: ");
    iter.Open(filename);

    // read and store all words
    
    timer.Start();
    for(iter.First(); ! iter.IsDone(); iter.Next())
    {
	word = iter.Current();
	ToLower(word);
        Update(wordList, WordCounter(word));
    }
    
    timer.Stop();    
    Print(wordList);    
    cout << "total # different words = " << wordList.Length() << endl;
    cout << "read and process time = " << timer.ElapsedTime() << endl;

    int occurs;
    MaxStats(wordList,word,occurs);
    cout << "max word = \"" << word << "\" # occurrences = "
			<< occurs << endl;

	 return 0;
}

void Print(const List<WordCounter> & wordList)
// postcondition: words and counts printed, one entry per line     
{
    for(wordList.First(); ! wordList.IsDone(); wordList.Next())
    {
        cout << wordList.Current().GetCount() << "\t";
        cout << wordList.Current().GetWord() << endl;
    }
}

void Update(List<WordCounter> & wordList, const WordCounter & wc)
// postcondition: word represented by wc is added to wordList if
//                it is not already in wordList.  Otherwise, the
//                count of wc.GetWord() is updated in wordList     
{
    for(wordList.First(); ! wordList.IsDone(); wordList.Next())
    {
        if (wordList.Current() == wc)
        {
            wordList.Current().Increment();
            return;
        }
    }
    wordList.Append(wc);
}

void MaxStats(const List<WordCounter> & wordList,
              apstring & maxWord, int & maxOccurrences)
// postcondition: maxWord = most frequently occurring word in wordList,
//                maxOccurrences = # times most frequent word occurs     
{
    maxOccurrences = 0;
    for(wordList.First(); ! wordList.IsDone(); wordList.Next())
    {
        if (wordList.Current().GetCount() > maxOccurrences)
        {
            maxOccurrences = wordList.Current().GetCount();
            maxWord = wordList.Current().GetWord();
        }
    }    
}

WordCounter::WordCounter()
  : myWord(""),
    myCount(0)
// postcondition: default wordcounter constructed
{
    // work done in initializer list
}

WordCounter::WordCounter(apstring word)
  : myWord(word),
    myCount(1)
// postcondition: word stored with count of 1     
{
    // work done in initializer list
}

void WordCounter::Increment()
// postcondition: # occurrences incremented by 1
{
    myCount++;
}

int WordCounter::GetCount() const
// postcondition: return # occurrences
{
    return myCount;
}

apstring WordCounter::GetWord() const
// postcondition: return word
{
    return myWord;
}

bool WordCounter::operator == (const WordCounter & rhs)
// postcondition: return true if word == rhs.word, # occurrences
//                is not used for determining equality     
{
    return myWord == rhs.myWord;
}
