#include <iostream.h>
#include <math.h>                    // for sqrt
#include <float.h>                   // for DBL_EPSILON
#include "prompt.h"

// Owen Astrachan, 6/9/96
// illustrates Newton Method's for square roots
// and floating point values/approximation

bool FloatEqual(double lhs, double rhs);
double Newton(double n);

int main()
{
	 int iterations = PromptRange("iterations:",1,30000);
    double low     = PromptRange("low:",0.0,1e30);
    double high    = PromptRange("high:",low,1e30);
    double delta   = (high - low)/iterations;
    double sroot,nroot;
    
    while (low <= high)        // loop from low to high, increment by delta
    {
        nroot = Newton(low);
        sroot = sqrt(low);

        if (!FloatEqual(sroot,nroot))
        {
            cout << "not float equal: " << low
                 << " newt = " << nroot << " sqrt = " << sroot << endl;     
        }
        else if (sroot != nroot)
        {
            cout << "not equal: " << low
                << " newt = " << nroot << " sqrt = " << sroot << endl;              
        }
        low += delta;
    }
    return 0;
}

bool FloatEqual(double lhs, double rhs)
// postcondition: returns true if lhs == rhs where equality
//                is determined by using relative error, i.e.
//                |lhs-rhs| / min(|lhs|,|rhs|)     
{
    const double EPSILON = DBL_EPSILON*10;  // precision for equality

    double flhs = fabs(lhs);          // compute absolute values
    double frhs = fabs(rhs);
    double min = frhs;
    if (flhs < frhs)                  // set minimum of absolute values
    {
        min = flhs;
    }
    if (min < EPSILON)                // avoid division by zero, check
    {
        return (fabs(lhs-rhs) < EPSILON);
    }
    return (fabs(lhs-rhs)/min < EPSILON);
}

double Newton(double n)
// precondition: n >= 0
// postcondition: returns sqrt(n)
{
    double root = n/2.0;
    
    if (FloatEqual(0.0,n))            // avoid division by zero
    {
        return 0.0;
    }
    
    while (!FloatEqual(root*root,n))  // loop until approximation is good
    {
        root = 0.5*(root + n/root);
    }
    return root;
}
