#include <iostream.h>
#include "apstring.h"

// a program to test linked list code
// use a struct for Node so that all fields are public

struct Node
{
    Node() : info(""),  next(0)              // default constructor
    { }
    Node(const apstring & val, Node * link=0)  // constructor
        : info(val), next(link)
    { }
    apstring info;                            // value stored
    Node * next;                            // link to next Node
};

void Append(Node* & list, const apstring & s)
// postcondition: new node with s is added to end of s
{
    if (list == 0)                   // empty list?
    {
        list = new Node(s);          // create first node
    }
    else
    {
        Node * temp = list;          // advance to end 
        while (temp->next != 0)
        {
            temp = temp->next;
        }
        temp->next = new Node(s);
    }
}

void Prepend(Node* & list, const apstring & s)
// postcondition: new node with s is added to front of s
{
    list = new Node(s,list);
}

void Delete(Node * & list, const apstring & s)
// postcondition: no node in list contains s,
//                nodes removed are deleted     
{
    if (list != 0)
    {
        Delete(list->next,s);
        if (list->info == s)
        {
            Node * temp = list->next;
            delete list;
            list = temp;
        }
    }
}

void Print(Node * list)
// postcondition: nodes of list printed, one per line
{
    cout << endl << "*****************" << endl << endl;
    while (list)
    {
        cout << "\t" << list->info << endl;
        list = list->next;
    }
    cout << endl << "*****************" << endl << endl;    
}

void Menu()
{
    cout << "(a) append to end of list"              << endl;
    cout << "(p) prepend to front of list"           << endl;
    cout << "(d) delete all occurrences from list"   << endl;
    cout << "(v) view/print the list"                << endl;
    cout << "(q) quit the program"                   << endl;
    cout << "(h) help, print choices"                << endl;
    cout << endl << "('a','p',d' prompt for a word)" << endl;
}

int main()
{
    apstring choice,word;
    Node * list = 0;
    char ch;
    
    Menu();
    do  
    {
        cout << "enter choice: ";
        cin >> choice;

        switch((ch = tolower(choice[0])))
        {
          case 'a' :
            cout << "enter word to append: ";
            cin >> word;
            Append(list,word);
            break;
          case 'p':
            cout << "enter word to prepend: ";
            cin >> word;
            Prepend(list,word);
            break;
          case 'd':
            cout << "enter word to delete: ";
            cin >> word;
            Delete(list,word);
            break;
          case 'v':
            Print(list);
            break;
          case 'h':
            Menu();
            break;
          case 'q':
            cout << "thanks for list testing" << endl;
            break;
          default:
            cout << "unrecognized choice: " << ch << endl;
        }
    } while (ch != 'q');
    return 0;
}

