#include <iostream.h>
#include "ctimer.h"

// illustrates timing code and comparing algorithms used
// for exponentiation
// Owen Astrachan

double LogPower(double base, int expo)
// precondition: expo >= 0     
// postcondition: returns base^expo (base to the power expo)
{
    if (0 == expo)
    {
        return 1.0;                             // correct for zeroth power
    }
    else
    {
        double semi = LogPower(base,expo/2);    // build answer from this
        
        if (expo % 2 == 0)                      // even exponent
        {
            return semi*semi;
        }
        else                                    // odd exponent
        {
            return base*semi*semi;
        }       
    }
}

double Power(double base,int expo)
// precondition:  expo >= 0
// postcondition: returns base^expo (base to the power expo)
{
    double result = 1;
    while (expo > 0)
    {
        result *= base;
        expo -= 1;
    }
    return result;
}

int main()
{
    int expo,iter;
    double base,result;
    
    cout << "enter base and exponent: ";
    cin >> base >> expo;

    cout << "enter number of iterations ";
    cin >> iter;
    
    CTimer timer;                      // used for timing code segments
    
    int k=0;
    while (k < iter)                   // time naive method
    {
        timer.Start();
        result = Power(base,expo);
        timer.Stop();
        k += 1;
    }

    cout << "time (linear) = " << timer.CumulativeTime() << endl;
    cout << "average = " << timer.CumulativeTime()/iter << endl;
    cout << "to compute " << base << "**" << expo << " = " << result << endl;

    timer.Reset();                    // reset to time next method
    k = 0;
    while (k < iter)                  // time logarithmic method
    {
        timer.Start();
        result = LogPower(base,expo);
        timer.Stop();
        k += 1;
    }

    cout << "time (logarithmic) = " << timer.CumulativeTime() << endl;
    cout << "average = " << timer.CumulativeTime()/iter << endl;
    cout << "to compute " << base << "**" << expo << " = " << result << endl;

    return 0;
}
