#include "walko.h"
#include "dice.h"

RandomWalkO::RandomWalkO(WalkObserver & obs, int maxSteps)
  : myObserver(obs)
// postcondition: all private data fields initialized     
{
    myPosition = mySteps = 0;
    myMaxSteps = maxSteps;
}

void RandomWalkO::TakeStep()
// postcondition: one step of random walk taken
{
    Dice coin(2);
    switch (coin.Roll())
    {
      case 1:
        myPosition--;
        break;
      case 2:
        myPosition++;
        break;
    }
    mySteps++;

    myObserver.Update(mySteps,myPosition);
}

void RandomWalkO::First()
// postcondition: first step of random walk taken
{
    myPosition = 0;
    mySteps = 0;
    TakeStep();
}

bool RandomWalkO::IsDone()
// postcondition: returns true when random walk is finished
//                i.e., when # of steps taken == max. # of steps     
{
    return mySteps >= myMaxSteps;
}

void RandomWalkO::Next()
// postcondition: next step in random walk simulated     
{
    TakeStep();
}

void RandomWalkO::Simulate()
{
    for(First(); ! IsDone(); Next())
    {
	// simulation complete using iterator methods
    }
}

int RandomWalkO::GetPosition()
// postcondition: returns position of "frog" (x coordinate)
{
    return myPosition;
}

int RandomWalkO::GetSteps()
// postcondition: returns number of steps taken by "frog"     
{
    return mySteps;
}
