// code from allword5.cc
// only the code that differs from allword4.cc
// is shown here, see allword5.cc for complete details
// 5-15-95

struct WordStat
{
    apstring info;                 // the string
    int count;                   // # of times string occurs
};


class WordList
{
  public:
    WordList(int size);             // constructor,  specify size of list
    ~WordList();                    // destructor, free storage
    void Update(apstring word);       // update occurrences of word
    int GetUnique();                // return # of unique words in list
    int GetTotal();                 // return # of total words in list
    void Print(ostream & output);   // print all words in list
  private:

    int myCount;                    // # of entries stored in myList
    int mySize;                     // capacity of myList
    int myTotal;                    // # of words (non-unique)
    apvector<WordStat *> myList;      // storage for word stats
    
    int Search(const apstring & key); // returns index in myList of key
};

WordList::WordList(int size)
   : myCount(0),
     mySize(size),
     myTotal(0),
     myList(size, NULL)
{
    // all work done in initializer list
}

WordList::~WordList()
// postcondition: all new'd storage deleted     
{
    int k;
    for(k=0; k < myCount; k++)
    {
        delete myList[k];
    }
}

void WordList::Update(apstring word)
// postcondition: updates occurrence of word
//                adds word to list if not already present
//                increments count of total # of words     
{
    int loc = Search(word);

    myTotal++;
    
    if (loc == -1)                 // not already in list
    {
        if (myCount >= mySize)
        {
            mySize *= 2;
            myList.resize(mySize);
        }
        loc = myCount - 1;
        while (0 <= loc && word < myList[loc]->info)
        {
            myList[loc+1] = myList[loc];
            loc--;
        }
        myList[loc+1] = new WordStat;
        myList[loc+1]->info = word;
        myList[loc+1]->count = 1;
        myCount++;
    }
    else
    {
        myList[loc]->count++;
    }
}
