// extracted from allword6.cc, just those parts
// that differ from allword5.cc

struct WordStat
{
    apstring info;                 // the string
    int count;                   // # of times string occurs
};


class WordList
{
  public:
    WordList(int size);             // constructor,  specify size of list
    ~WordList();                    // destructor, free storage
    void Update(apstring word);       // update occurrences of word
    int GetUnique();                // return # of unique words in list
    int GetTotal();                 // return # of total words in list
    void Print(ostream & output);   // print all words in list
    void NumPrint(ostream & output);// print all words in list (by # occs)
  private:

    int myCount;                    // # of entries stored in myList
    int mySize;                     // capacity of myList
    int myTotal;                    // # of words (non-unique)
    
    apvector<WordStat *> myList;      // storage for word statistics
    apvector<WordStat *> myNumList;   // for sorting by # occurrences
    
    int Search(const apstring & key); // returns index in myList of key
    void DoPrint(ostream & output,const apvector<WordStat *> & list);
    void Sort();                    // sort by number
};

WordList::WordList(int size)
   : myCount(0),
     mySize(size),
     myTotal(0),
     myList(size),
     myNumList(0)
{
    // all work done in initializer list
}

void WordList::Sort()
// postcondition: list can be printed sorted by # of occurrences     
{
    int j,k,min;

    myNumList.resize(myCount);        // allocate space for pointers
    for(j=0; j < myCount; j++)         // copy pointers to all words
    {
        myNumList[j] = myList[j];
    }
    
    for(j=0; j < myCount; j++)         // selection sort by # occurrences
    {
        min = j;
        for(k=j+1; k < myCount; k++)
        {
            if (myNumList[k]->count < myNumList[min]->count)
            {
                min = k;
            }
        }
        WordStat * temp = myNumList[min];
        myNumList[min] = myNumList[j];
        myNumList[j] = temp;
    }
}

void WordList::Print(ostream & output)
// postcondition: all elements of list printed, sorted alphabetically     
{
    DoPrint(output,myList);
}

void WordList::NumPrint(ostream & output)
// postcondition: all elements of list printed, sorted by # occurrences
{
    Sort();
    DoPrint(output,myNumList);
}

int main()
{
    // code here removed (see allword6.cc)
    
    timer.Start();
    wlist.NumPrint(cout);
    timer.Stop();
    
    cout << "total words (non-unique): " << wlist.GetTotal() << endl;
    cout << "total time for update = " << processTime << endl;
    cout << "total time for sorting = " << timer.ElapsedTime() << endl;
    return 0;
}
