
// extracted from allword7.cc, just those parts
// that show important aspects of linked list code

struct WordStat
{
    apstring info;                 // the string
    int count;                   // # of times string occurs
    WordStat * next;             // pointer to next node in list
};

class WordList
{
  public:
    WordList();                     // constructor
    ~WordList();                    // destructor
    void Update(const apstring & word);  // update occurrences of word
    int GetUnique();                // return # of unique words in list
    int GetTotal();                 // return # of total words in list
    void Print(ostream & output);   // print all words in list
  private:

    int myCount;                    // # of entries stored in myList
    int myTotal;                    // # of words (non-unique)
    WordStat * myList;              // pointer to first node of list
    
    WordStat * Search(const apstring & key); // returns pointer
};

WordList::~WordList()
// postcondition: all new'd storage deleted     
{
    WordStat * current = myList;
    while (current != 0)
    {
        myList = myList->next;       // advance to next
        delete current;              // delete current (first)
        current = myList;            // reset current
    }
}

WordStat * WordList::Search(const apstring & key)
// postcondition: returns pointer to struct with key
//                returns 0 if key does not occur
// performance: O(n), n = # items in myList                    
{
    WordStat * current = myList;
    while (current != 0)
    {
        if (current->info == key)
        {
            return current;
        }
        current = current->next;
    }
    return 0;
}

void WordList::Update(const apstring & word)
// postcondition: updates occurrence of word
//                adds word to list if not already present
//                otherwise updates count of # occurrences     
//                increments count of total # of words     
{
    WordStat * loc = Search(word);
    myTotal++;

    if (loc != 0)
    {
        loc->count++;
    }
    else
    {
        myCount++;
        WordStat * wptr = new WordStat; // create new node
        wptr->info  = word;             // store values
        wptr->count = 1;                // initialize all fields
        wptr->next  = 0;
        
        
        if (myList == 0)                // any nodes at all?
        {
            myList = wptr;
        }
        else if (word < myList->info)   // goes in front?
        {
            wptr->next = myList;
            myList = wptr;
        }
        else                            // find node before new word
        {
            loc = myList;
            while (loc->next && loc->next->info < word)
            {
                loc = loc->next;
            }
            wptr->next = loc->next;
            loc->next = wptr;
        }
    }
}
