#ifndef _WORDSTREAMITERATOR_H
#define _WORDSTREAMITERATOR_H

// Owen Astrachan 7/3/95
//
// class WordStreamIterator
//
// void Open(apstring name)
//        -- initializes iterator to file specified by name
//
// void First(), void Next(), bool IsDone()
//        -- "standard" iterating functions (see below)
//
// usage: call First(), before accessing Current()
//        call Next() to move to  the next word in the stream
//        call Current() to access the current word
//
// apstring Current()
//        -- returns current apstring (see below)
//
// words are retrieved from stream until sentinel LAST_WORD is encountered
// this sentinel word is read from the stream.  Once read, IsDone()
// will return true.
//
// properties:
//
// if IsDone() == false, then accessing the current word
//   is a valid operation; access is provided by the member
//   function Current()
//
// if IsDone() == true, then accessing the current word is
//   NOT a valid operation
//
//

#include "apstring.h"
#include <fstream.h>

class WordStreamIterator
{
  public:
    WordStreamIterator();
    void Open(apstring name);           // bind stream to specific text file
    void First();                     // initialize iterator
    apstring Current();                 // returns current word
    bool IsDone();                    // true if iterator is done
    void Next();                      // advance to next word

  private:
    apstring myWord;                    // the current word
    bool myDone;                      // true if no more words
    ifstream myInput;                 // the stream to read from
};

#endif
